#!/bin/bash
#
# Copyright (c) 2021-2023, AlterBasics Technologies Pvt Ltd.
#
# All Rights Reserved.
# You may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
# https://colrows.com/license.txt
#

function showUsage {
  echo -e "About:"
  echo -e "\tthis script will start and stop colrows server"
  echo -e "\nUsage:"
  echo -e "\t$0 [COMMAND] [OPTION]"
  echo -e "\nCommands:"
  echo -e "\tstart   : perform a backup without re-configuring the system"
  echo -e "\tstop    : setup ssh and perform a backup without re-configuring the system"
  echo -e "\t-help   : show script uses"
  echo -e "\nOptions:"
  echo -e "\t--server   : start colrows as clustered server "
  echo -e "\t--http     : start colrows as a single instance server"
  echo -e "\t--cluster  : strat colrows cluster"
  echo -e "\t--experimental : strat colrows  as experimental server with dummy data [for single instance / default]"
  echo -e "\nExamples:"
  echo -e "\t$0 start  (default it will start as server with clustering)"
  echo -e "\t$0 start --server"
  echo -e "\t$0 start --http"
  echo -e "\t$0 start --cluster"
  echo -e "\t$0 start --experimental"
  echo -e "\t$0 stop"
}

if [ $# -eq 0 ]; then
  showUsage
  exit 1
fi

if [ "$1" == "start" ]; then
  pushd ../
  if [ -z "$2" ] || [ "$2" == "--server" ]; then
    bootseq='SERVER'
  elif [ "$2" == "--http" ]; then
    bootseq='HTTP'
  elif [ "$2" == "--cluster" ]; then
    bootseq='CLUSTER'
  elif [ "$2" == "--experimental" ]; then
    bootseq='EXPERIMENTAL'
  else
    echo "Invalid argument $2"
    showUsage
    exit 1
  fi

  ## start colrows process
  echo "starting colrows process"
  app=$(ls ./lib/colrows-server*.jar)
  echo "Deploying application jar : $app"

  if [ -z $JAVA_HOME ]; then
    # Attempt to find the Java path
    JAVA_PATH=$(readlink -f $(which java))
    if [ -z "$JAVA_PATH" ]; then
      echo "Java is not installed or not in the PATH. Please consider setting JAVA_HOME if java is already installed"
      exit 1
    fi
    # Deduce JAVA_HOME from JAVA_PATH and update it
    JAVA_HOME=$(dirname $(dirname "$JAVA_PATH"))

    # Print out JAVA_HOME for user's reference
    echo "Detected JAVA_HOME: $JAVA_HOME"

    # Set JAVA_HOME for the current session
    export JAVA_HOME

  fi
  jvmOptions=$(cat conf/jvm.options | tr -s '\n' ' ')
  echo "JVM Options: $jvmOptions"
  echo "Clustering option :$bootseq"

  #get current HOME location and set it to COLROWS_HOME
  # MAKE SURE COLROWS_HOME is last argument in program run command
  #it would be further use appliction to detect home location on STOP command

  # Get the full path to the directory containing the script
  COLROWS_HOME=$(cd "$(dirname "${BASH_SOURCE[0]}")" &>/dev/null && pwd)
  echo "Colrows home dir :$COLROWS_HOME"

  nohup $JAVA_HOME/bin/java $jvmOptions -cp "$app:./lib/*" com.colrows.bootstrap.Colrows -bootseq $bootseq -COLROWS_HOME $COLROWS_HOME >nohup.out &
  tail -f nohup.out
  echo "Application started successfully"
  exit 0
elif [ "$1" == "stop" ]; then
  export counter=0
  # Using ps, grep to filter processes, and excluding grep process itself
  ps -ef | grep "colrows" | grep -v grep | grep -v colrows.sh | (
    while read line; do
      ((counter++))
      PID=$(echo "$line" | awk '{print $2}')
      CMD=$(echo "$line" | awk '{print $NF}')

      COLROWS_HOME_VALUE=$(echo "$line" | awk '{print $NF}')
      SCRIPT_DIR=$(dirname "$(readlink -f "$0")")
      echo "checking colrows root location"
      # Check if mainString contains subString using grep

      if expr "$SCRIPT_DIR" : ".*$COLROWS_HOME_VALUE.*" >/dev/null; then
        # Try to kill the process
        kill $PID 2>/dev/null
        # Check if the kill command was successful
        if [ $? -eq 0 ]; then
          echo "Process $PID has been terminated."
        else
          echo "Failed to terminate process $PID. It may not exist, or you might lack the necessary permissions."
        fi
      fi
    done
    echo $counter >temp_counter.txt
  )
  counter=$(<temp_counter.txt)
  if [ "$counter" -eq 0 ]; then
    echo "No Running Colrows instance found"
  fi

  exit 0
elif [ "$1" == "-help" ]; then
  showUsage
  exit 0
else
  echo "Required arguments missing"
  showUsage
  exit 1
fi
